/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2013  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.88e                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit.c   (for Renesas M32C and NC308 compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your
          target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.h"

/*********************************************************************
*
*        SFRs
*
**********************************************************************
*/
#define TA0      (*(volatile OS_U16*) 0x000346)  // Timer A0 register
#define TA0MR    (*(volatile OS_U8* ) 0x000356)  // Timer A0 mode register
#define TA0IC    (*(volatile OS_U8* ) 0x00006C)  // Timer A0 interrupt control register
#define TABSR    (*(volatile OS_U8* ) 0x000340)  // Count start flag
#define PRCR     (*(volatile OS_U8* ) 0x00000A)  // Protect register
#define MCD      (*(volatile OS_U8* ) 0x00000C)  // Main clock division register
#define U0SMR    (*(volatile OS_U8* ) 0x000367)  // UART0 special mode register
#define U0SMR2   (*(volatile OS_U8* ) 0x000366)  // UART0 special mode register 2
#define U0SMR3   (*(volatile OS_U8* ) 0x000365)  // UART0 special mode register 3
#define U0SMR4   (*(volatile OS_U8* ) 0x000364)  // UART0 special mode register 4
#define U0MR     (*(volatile OS_U8* ) 0x000368)  // UART0 transmit-receive mode register
#define U0BRG    (*(volatile OS_U8* ) 0x000369)  // UART0 bit rate generator
#define U0C0     (*(volatile OS_U8* ) 0x00036C)  // UART0 transmit-receive control register 0
#define U0C1     (*(volatile OS_U8* ) 0x00036D)  // UART0 transmit-receive control register 1
#define U0TB     (*(volatile OS_U16*) 0x00036A)  // UART0 transmit buffer register
#define U0RB     (*(volatile OS_U16*) 0x00036E)  // UART0 receive buffer register
#define S0TIC    (*(volatile OS_U8* ) 0x000090)  // UART0 transmit/NACK interrupt control register
#define S0RIC    (*(volatile OS_U8* ) 0x000072)  // UART0 receive/ACK interrupt control register
#define P6       (*(volatile OS_U8* ) 0x0003C0)  // Port P6 register
#define PD6      (*(volatile OS_U8* ) 0x0003C2)  // Port P6 direction register
#define PS0      (*(volatile OS_U8* ) 0x0003B0)  // Function select register A0

/*********************************************************************
*
*        Configuration
*
**********************************************************************
*/

#ifndef OS_FSYS
  #define OS_FSYS 24000000L
#endif

/* Select UART for embOSView.
   If you do not want (or can not due to hardware limitations) to dedicate
   a UART to embOSView, please define it to be -1
   Currently the standard code below supports only UART 0 and 1
   (But is easily changed to other UARTs)
*/
#ifndef OS_UART
  #define OS_UART 0
#endif    
#ifndef OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*        OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {
  TA0 = (OS_FSYS/1000-1);  // Generate Interrupts every 1 ms
  TA0MR=0;        // Timer mode register
                  // bit 01 : 00 Timer mode
                  // bit 2  :  0 No puls output
                  // bit 34 : 0X No gate function
                  // bit 5  : 0
                  // bit 67 : 00 clock source : fx/2
  TA0IC=1;        // Timer interrupt register, priority 1 (lowest)
  TABSR |= 1;     // Set Timer count start flag
  PRCR  = 0x03;   // Enable write to Processor mode
  MCD   = 0x12;   // main clock division register: no division
  PRCR  = 0x00;   // Disable access to processor and clock mode
  OS_COM_Init();  // Initialize UART for embOSView
}

/*********************************************************************
*
*        Idle loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not run as task, soit does not have have a stack of its own,
 therefore no functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {    // Idle loop: No task is ready to exec
  for (;;);             // Nothing to do ... wait for interrupt
}

/*********************************************************************
*
*        Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer.
    It returns the system time in clock cycles.
*/

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = TA0;
  OS_U32 time = OS_Time+1;
  //
  // Check if timer interrupt pending ...
  if (TA0IC & (1 << 3)) {
    t_cnt = TA0;
    time++;
  }
  return (OS_FSYS/1000) * time - t_cnt;
}

/*********************************************************************
*
*        OS_ConvertCycles2us
*
**********************************************************************

   Convert Cycles into micro seconds. When using 20.0 MHz, this
   means dividing by 20. If you have a different clock frequency,
   you may have to modify this routine in order to get proper
   diagonstics. Please note: Operation of embOS is not affected
   by this routine. It is used for profiling and testing
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*        OS_Tick interrupt
*
**********************************************************************
*/

#pragma INTERRUPT OS_ISR_Tick
void OS_ISR_Tick (void) { 
  OS_EnterNestableInterrupt();
  OS_TICK_Handle();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*        Communication for embOSView   (UART 0)
*
**********************************************************************
*/

#define BAUDDIVIDE ((OS_FSYS+(OS_BAUDRATE*8L))/(OS_BAUDRATE*16L))-1
#if (OS_UART == 0)

void OS_COM_Init(void) {
  U0MR   = 0x00;        // lock Sio, error reset
  U0C0   = 0x10;        // RTS/CTS disabled, clock divisor 1
  U0BRG  = BAUDDIVIDE;  // Calculated Baudrate
  U0C1   = 0;           // Lock Rx and Tx
  U0MR   = 0x05         // 8  Data
          +(0 << 4)     // 0: 1 Stop-Bit (1= 2 Stop-Bits)
          +(0 << 5)     // X: Parity select
          +(0 << 6)     // 1: parity enable
          +(0 << 7);    // 0: no sleep
  U0C1   = 5;           // enable reception and transmition
  S0RIC  = 1;           // Enable UART Rx interrupts
  S0TIC  = 1;           // Enable UART Tx interrupts
  PD6   |= (1 << 3);    // Direction register   (0=IN 1=OUT)
  P6    |= (1 << 3);    // Register  (0=Low, 1= High)
  PS0   |= (1 << 3);    // Enable Tx0
}

void OS_COM_Send1(OS_U8 c) {
  U0TB = c;
}

/* The interrupt routines */

#pragma INTERRUPT OS_ISR_tx
void OS_ISR_tx (void) {
  OS_EnterNestableInterrupt();
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

#pragma INTERRUPT OS_ISR_rx
void OS_ISR_rx (void) {
  int Data;
  OS_EnterNestableInterrupt();  //  We will enable interrupts 
  Data = U0RB;
  if (Data & 0x6000) {   /* Check if errors occurred */
    U0C1 &= ~(1 << 2);   // disable Rx
    U0C1 |=  (1 << 2);   // enable Rx
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*        Communication for embOSView   (UART 1)
*
**********************************************************************
*/

#elif (OS_UART == 1)

void OS_COM_Init(void) {
  U1MR   = 0x00;        // lock Sio, error reset
  U1C0   = 0x10;        // RTS/CTS disabled, clock divisor 1
  U1BRG  = BAUDDIVIDE;  // Calculated Baudrate
  U1C1   = 0;           // Lock Rx and Tx
  U1MR   = 0x05         // 8  Data
          +(0 << 4)     // 0: 1 Stop-Bit (1= 2 Stop-Bits)
          +(0 << 5)     // X: Parity select
          +(0 << 6)     // 1: parity enable
          +(0 << 7);    // 0: no sleep
  U1C1   = 5;           // enable reception and transmition
  S1RIC  = 1;           // Enable UART Rx interrupts
  S1TIC  = 1;           // Enable UART Tx interrupts
  PD6  |= (1 << 7);     // Direction register   (0=IN 1=OUT)
  P6   |= (1 << 7);     // Register  (0=Low, 1= High)
  PS0  |= (1 << 7);     // Enable Tx1
}

void OS_COM_Send1(OS_U8 c) {
  U1TB = c;
}

/* The interrupt routines */
#pragma INTERRUPT OS_ISR_tx
void OS_ISR_tx (void) {
  OS_EnterNestableInterrupt();
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

#pragma INTERRUPT OS_ISR_rx
void OS_ISR_rx (void) {
  int Data;
  OS_EnterNestableInterrupt();  //  We will enable interrupts 
  Data = U1RB;
  if (Data & 0x6000) {  // Check if errors occurred
    U1C1 &= ~(1 << 2);  // disable Rx
    U1C1 |=  (1 << 2);  // enable Rx
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

#else  /* No communication routines */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* let OS know that transmitter is not busy  */
  }
#pragma INTERRUPT OS_ISR_tx
  void OS_ISR_tx(void) {}
#pragma INTERRUPT OS_ISR_rx
  void OS_ISR_rx(void) {}
#endif /* (OS_UART == 0 ) || (OS_UART == 1) */

/****** End Of File *************************************************/

